use chrono::{Local, NaiveDate, NaiveTime};
use sqlx::{query, PgPool};

use super::Result;

pub struct VehicleAssignement {
    pub event_id: i32,
    pub vehicle_id: i32,
    pub start_time: NaiveTime,
    pub end_time: NaiveTime,
}

impl VehicleAssignement {
    pub async fn create(
        pool: &PgPool,
        event_id: i32,
        vehicle_id: i32,
        start_time: NaiveTime,
        end_time: NaiveTime,
    ) -> Result<()> {
        // TODO: refactor
        let date1 = Local::now();
        let date2 = Local::now();

        query!(
            "INSERT INTO vehicleassignement (eventId, vehicleId, startTimestamp, endTimestamp) VALUES ($1, $2, $3, $4);",
            event_id,
            vehicle_id,
            date1,
            date2
        )
        .execute(pool)
        .await?;

        Ok(())
    }

    pub async fn read(
        pool: &PgPool,
        event_id: i32,
        vehicle_id: i32,
    ) -> Result<Option<VehicleAssignement>> {
        let record = query!("SELECT * FROM vehicleAssignement WHERE vehicleAssignement.eventId = $1 AND vehicleAssignement.vehicleId = $2;", event_id, vehicle_id).fetch_optional(pool)
            .await?;

        // TODO: refactor
        let vehicle_assignment = record.map(|r| VehicleAssignement {
                event_id: r.eventid,
                vehicle_id: r.vehicleid,
                start_time: r.starttimestamp.time(),
                end_time: r.endtimestamp.time(),
        });

        Ok(vehicle_assignment)
    }

    pub async fn read_all_by_event(
        pool: &PgPool,
        event_id: i32,
    ) -> Result<Vec<VehicleAssignement>> {
        let records = query!(
            "SELECT * FROM vehicleAssignement WHERE vehicleAssignement.eventId = $1;",
            event_id
        )
        .fetch_all(pool)
        .await?;

        // TODO: refactor
        let vehicle_assignments = records
            .iter()
            .map(|r| VehicleAssignement {
                event_id: r.eventid,
                vehicle_id: r.vehicleid,
                start_time: r.starttimestamp.time(),
                end_time: r.endtimestamp.time(),
            })
            .collect();

        Ok(vehicle_assignments)
    }

    pub async fn read_all_by_vehicle_and_date(
        pool: &PgPool,
        vehicle_id: i32,
        date: NaiveDate
    ) -> Result<Vec<VehicleAssignement>> {
        let records = query!(
            r#"
            SELECT
                vehicleAssignement.eventId,
                vehicleAssignement.vehicleId,
                vehicleAssignement.starttimestamp,
                vehicleAssignement.endtimestamp
            FROM vehicleAssignement
            JOIN event ON vehicleAssignement.eventId = event.id
            WHERE vehicleAssignement.vehicleid = $1
            AND event.starttimestamp::date = $2;
            "#,
            vehicle_id,
            date,
        )
        .fetch_all(pool)
        .await?;

        // TODO: maybe ::date doesnt work

        let vehicle_assignments = records
            .iter()
            .map(|r| VehicleAssignement {
                event_id: r.eventid,
                vehicle_id: r.vehicleid,
                start_time: r.starttimestamp.time(),
                end_time: r.endtimestamp.time(),
            })
            .collect();

        Ok(vehicle_assignments)
    }

    pub async fn delete(pool: &PgPool, event_id: i32, vehicle_id: i32) -> Result<()> {
        query!(
            "DELETE FROM vehicleassignement WHERE eventId = $1 AND vehicleId = $2;",
            event_id,
            vehicle_id
        )
        .execute(pool)
        .await?;

        Ok(())
    }
}
